import { PaymentMethodToken, ResumeToken, SuccessCallbackReturnType } from '@primer-io/checkout-web'
import Vue, { PropType } from 'vue'
import Component from 'vue-class-component'
import {Watch} from 'vue-property-decorator'



type PromiseResolver = {
  resolve: (value: unknown) => void;
  reject: (err: Error) => void;
}


///////////////////////////////////////////
// onTokenizeSuccess
///////////////////////////////////////////
const onTokenizeSuccessMixinProps = Vue.extend({
  props: {
    tokenizeSuccessStatus: {
      type: Object as PropType<SuccessCallbackReturnType | Boolean | Error> ,
      required: false
    },

    tokenizeSuccess: {
      type: Function,
      required: false
    }
  },
})
@Component
export class onTokenizeSuccessMixin extends onTokenizeSuccessMixinProps {
  onTokenizeSuccessPromise: PromiseResolver | undefined = undefined
  clearOnTokenizeSuccess() {
    this.onTokenizeSuccessPromise = undefined
    this.$emit('tokenize-success-finish')
  }

  isWaitingForOnTokenizeSuccess() {
    return !!this.onTokenizeSuccessPromise
  }

  resolveOnTokenizeSuccess(data: Error | SuccessCallbackReturnType) {
    if (!this.isWaitingForOnTokenizeSuccess()) {
      return
    }

    if (data instanceof Error) {
      this.onTokenizeSuccessPromise?.reject(data)
    } else {
      this.onTokenizeSuccessPromise?.resolve(data)

    }

    this.clearOnTokenizeSuccess()
  }

  onTokenizeSuccess(arg: PaymentMethodToken) {
    // Using function props
    if (this.tokenizeSuccess) {
      return this.tokenizeSuccess(arg)
    }

    // Promise to return
    const promise = new Promise((resolve, reject) => {
      this.onTokenizeSuccessPromise = {resolve, reject}
    })

    // Create callback to pass to event
    const next = (data: Error | SuccessCallbackReturnType) => {
      this.resolveOnTokenizeSuccess(data)
    }

    this.$emit('tokenize-success', arg, next)

    return promise
  }

  @Watch('tokenizeSuccessStatus')
  onTokenizeSuccessStatusChange(tokenizeSuccessStatus: Error | SuccessCallbackReturnType) {
    if (!this.isWaitingForOnTokenizeSuccess()) {
      return
    }

    if (tokenizeSuccessStatus instanceof Error) {
      this.onTokenizeSuccessPromise?.reject(tokenizeSuccessStatus)
      this.clearOnTokenizeSuccess()
    } else if (tokenizeSuccessStatus) {
      this.onTokenizeSuccessPromise?.resolve(tokenizeSuccessStatus)
      this.clearOnTokenizeSuccess()
    }
  }

}

///////////////////////////////////////////
// onResumeSuccess
///////////////////////////////////////////
const onResumeSuccessMixinProps = Vue.extend({
  props: {
    resumeSuccessStatus: {
      type: [Boolean, Error],
      required: false
    },

    resumeSuccess: {
      type: Function,
      required: false
    }
  },
})
@Component
export class onResumeSuccessMixin extends onResumeSuccessMixinProps {
  onResumeSuccessPromise: PromiseResolver | undefined = undefined

  clearOnResumeSuccess() {
    this.onResumeSuccessPromise = undefined
    this.$emit('resume-success-finish')
  }

  isWaitingForOnResumeSuccess() {
    return !!this.onResumeSuccessPromise
  }

  resolveOnResumeSuccess(data: Error | SuccessCallbackReturnType) {
    if (!this.isWaitingForOnResumeSuccess()) {
      return
    }

    if (data instanceof Error) {
      this.onResumeSuccessPromise?.reject(data)
    } else  {
      this.onResumeSuccessPromise?.resolve(data)
    }

    this.clearOnResumeSuccess()
  }

  onResumeSuccess(arg: ResumeToken) {
    // Using function props
    if (this.resumeSuccess) {
      return this.resumeSuccess(arg)
    }

    // Promise to return
    const promise = new Promise((resolve, reject) => {
      this.onResumeSuccessPromise = {resolve, reject}
    })

    // Create callback to pass to event
    const next = (data: Error | SuccessCallbackReturnType) => {
      this.resolveOnResumeSuccess(data)
    }

    this.$emit('resume-success', arg, next)

    return promise
  }

  @Watch('resumeSuccessStatus')
  onResumeSuccessStatusChange(resumeSuccessStatus: Error | SuccessCallbackReturnType) {
    if (!this.isWaitingForOnResumeSuccess()) {
      return
    }

    if (resumeSuccessStatus instanceof Error) {
      this.onResumeSuccessPromise?.reject(resumeSuccessStatus)
      this.clearOnResumeSuccess()
    } else {
      this.onResumeSuccessPromise?.resolve(resumeSuccessStatus)
      this.clearOnResumeSuccess()
    }
  }

}
